<?

/**
 * Input with a check for host name validity in a DNS zone. No dot allowed !
 */
class HostnameInputTpl extends InputTpl {

    function HostnameInputTpl($name) {
        $this->InputTpl($name, '/^[a-z][a-z0-9-]*[a-z0-9]$/');
    }

}

/**
 * Input with a basic check for a colon separated list of host name and/or IP
 */
class HostIpListInputTpl extends InputTpl {
    
    function HostIpListInputTpl($name) {
        $this->InputTpl($name, '/^[0-9a-z,.-]*$/');
    }

}

/**
 * Input with a check for a valid net mask (range from 1 to 32)
 */
class NetmaskInputTpl extends InputTpl {

    function NetmaskInputTpl($name) {
        $this->InputTpl($name, '/^([1-9]|[1-2][0-9]|3[0-2])$/');
    }

}

/**
 * Input with a check for a valid simple net mask (8, 16, or 24)
 */
class SimpleNetmaskInputTpl extends InputTpl {

    function SimpleNetmaskInputTpl($name) {
        $this->InputTpl($name, '/^(8|16|24)$/');
    }

}

/**
 * Input with a check for a valid MX record (eg "10 smtp.example.net")
 */
class MXRecordInputTpl extends InputTpl {

    function MXRecordInputTpl($name) {
        $this->InputTpl($name, '/^([1-9]|[1-9][0-9]) ([a-z][a-z0-9-]*[a-z0-9]\.){0,10}[a-z][a-z0-9-]*[a-z0-9]$/');
    }

}


class GetFreeIPInputTpl extends IPInputTpl {

    function getFreeIPInputTpl() {
        parent::IPInputTpl("ipaddress");        
    }

    function display($arrParam) {
        parent::display($arrParam);
        $zone = $arrParam["zone"];
        $ajaxurl = $arrParam["ajaxurl"];
        if (empty($arrParam["subnet"])) {
            $subnet = '';
        } else {
            $subnet = $arrParam["subnet"];
        }
        print '

<script type="text/javascript">

function setZoneFreeAddress(t) {
    $("ipaddress").value = t.responseText;
    if (! ($("ipaddress").validate()) ) $("ipaddress").value = "";
    new Effect.Highlight("ipaddress",
    { duration: 1.0 });
}

</script>

<input type="button" class="btnPrimary" onclick="new Ajax.Request(\'main.php?module=network&submod=network&action=' . $ajaxurl . '\', {method: \'get\', parameters: \'zone=' . $zone . '&subnet=' . $subnet . '&current=\' + $F(\'ipaddress\'), onSuccess:setZoneFreeAddress});" value="' . _T("Get next free IP address") . '">
';

    }
}

/* Ip functions */

/**
 * Return True if an IP is in a network
 * 
 * If $acceptBoundaries is True, the base network address and the broadcast address are accepted
 */
function ipInNetwork($ip, $network, $mask, $acceptBoundaries = False) {
    $ip = ip2long($ip);
    $network = ip2long($network);
    $mask = intval($mask);
    if (!$acceptBoundaries) {
        if ($ip == $network) return False; /* Network address */
        $ipmask = ip2long("255.255.255.255") << (32 - $mask);
        if ($ip == ($network | (~$ipmask))) return False; /* Broadcast address */
    }
    $ret = True;
    for ($i = 0 ; $i < $mask ; $i++) {
        $n = pow(2, 31 - $i) ;
        if (($n & $ip) != ($n & $network)) {
            $ret = False;
            break;
        }
    }    
    return $ret;
}

/**
 * Return True if IP A is lower than IP B
 */
function ipLowerThan($ipa, $ipb) {
    return ip2long($ipa) < ip2long($ipb);
}

/**
 * Return True if IP is into a IP range
 */
function ipInRange($ip, $ipa, $ipb) {
    return (ip2long($ipa) <= ip2long($ip)) and (ip2long($ip) <= ip2long($ipb));
}

/* Some common used utility functions */

function getSubnetOptions($subnet) {
    $options = array();
    if (isset($subnet[0][1]["dhcpOption"])) {
        foreach($subnet[0][1]["dhcpOption"] as $option) {
            list($name, $value) = explode(" ", $option, 2);
            $options[$name] = trim($value, '"');
        }
    }
    if (isset($options["domain-name"])) {
        $tmp = explode(" ", $options["domain-name"]);
        $options["primarydomainname"] = $tmp[0];
    }    
    return $options;
}

function getSubnetStatements($subnet) {
    $statements = array();
    if (isset($subnet[0][1]["dhcpStatements"])) {
        foreach($subnet[0][1]["dhcpStatements"] as $statement) {
            list($name, $value) = explode(" ", $statement, 2);
            $statements[$name] = trim($value, '"');
        }
    }
    return $statements;
}

function isAuthoritative($subnet) {
    $ret = False;
    $statements = array();
    if (isset($subnet[0][1]["dhcpStatements"])) {
        foreach($subnet[0][1]["dhcpStatements"] as $statement) {
            if ($statement == "authoritative") {
                $ret = True;
                break;
            }
        }
    }
    return $ret;
}

/* Check that the given subnet is not contained into an existing subnet */
function checkSubnet($subnet) {
    foreach(getSubnets("") as $dn => $entry) {
        $sub = $entry[1]["cn"][0];
        $mask = $entry[1]["dhcpNetMask"][0];
        if (ipInNetwork($subnet, $sub, $mask, True)) {
            $errormessage = sprintf(_T("The given network address belongs to the already existing DHCP subnet %s / %s."), $sub, $mask);
            break;
        }
    }
    return array(isset($errormessage), $errormessage);
}

/* Check that the given pool range is valid */
function checkPools($subnet, $netmask, $hasPools, $pools=null) {
    if ($hasPools) {
        for ($i = 0; $i < count($pools); $i++)
        {
            if (strlen(trim($pools[$i]))>0) {
                list($ipStart, $ipEnd) = split(" ", $pools[$i]);
                if (isset($ipStart) && isset($ipEnd)) {
                    if (!(ipLowerThan($ipStart, $ipEnd) && ipInNetwork($ipStart, $subnet, $netmask) && ipInNetwork($ipEnd, $subnet, $netmask))){
                        $errormessage .= sprintf(_T("The specified dynamic pool IP range from %s to %s is not valid."), $ipStart, $ipEnd);
                        if ($i < count($pools) -1) $errormessage .= "<br>";
                    }
                }
            }
            if (!isset($pools[$i]) or (strlen(trim($pools[$i]))==0) or !isset($ipStart) or !isset($ipEnd)) {
                    $errormessage .= sprintf(_T("No IP range specified for %d dynamic pool."), $i + 1);
                    if ($i < count($pools) -1) $errormessage .= "<br>";
            }
        }
    }
    return array(isset($errormessage), $errormessage);
}

/* Check if the given combination of subnet and mask is valid */
function checkSubnetNetmask($subnet, $netmask) {
    $ipsubnet=ip2long($subnet);
    $ipmask = ip2long("255.255.255.255") << (32 - $netmask);
    if (($ipsubnet & $ipmask) != ($ipsubnet)) {
        $errormessage = sprintf(_T("Subnet number/mask combination is not valid."));
    }
    return array(isset($errormessage), $errormessage);
}

?>
